/*	Replication do-file
	Jones and Metzger, "Evaluating Conflict Dynamics", JCR
	
	02FEB16
	
	
	DESCRIPTION:
		This file provides replication code to generate the paper's dataset, and to
		generate all figures and tables in the paper.  If the relevant table/figure
		is generated in R, this file notes as much and gives you the R file's name.		
		
	NOTE: 
		Unique identifier for dispute-dyads = claimDySwID					
		
	REQUIRED PACKAGES:				
		* Clarify (net from http://gking.harvard.edu/clarify)
		* modes (net sj 9-4 sg113_2)
		* five user-written ado files (copy everything from "assembly files/ado" in replication folder to local ado folder)
			// To find loc of local ado folder, type: di "`c(sysdir_personal)'"												*/
			
******************** MUST SET ********************

global dir "C:/Users/Shawna/Desktop/JCR replic/assembly files/"	// to define the local working directory for *assembly files*
															* note: both PC and Mac need forward slash at end of path. 
																														
// ^ 	Run above line.  Each code block is written independently, and can then 
//		be run by highlighting the entire relevant block and exectuting.


*******************************************************************************	
// DATASET ASSEMBLY
**************************
cd "$dir"
cd ..
do "assembly files/__master (use me!).do"
	* NOTE: generating the dataset will take 10-15 minutes.


*******************************************************************************	
// TABLES
**************************
// Table 1 - Observed Transition Frequencies (four stages)
{
	cd "$dir"
	cd ..
	use "2 - h&a - continuous, no TVC.dta", clear
	
	// numerator
	tab stage nextStage if(status==1)	
	
	// denominator for percentages
	qui{
		cap drop temp
		bysort claimDySwID t0 (nextStage): gen temp = _n==1
		forvalues s=0/3{
			noi di _n as gr "Stage " `s'
			noi count if(temp==1 & stage==`s')
			local denom = r(N)
			
			count if(status==1 & stage==`s')
			noi di as gr "TOTAL Col %: " as ye r(N)/`denom'*100
		}
		drop temp
	}
}
**************************
// Table 2 - Top 5 Transition Sequences by Frequency
{
	cd "$dir"
	cd ..
	use "2 - h&a - continuous, no TVC.dta", clear

	qui{
		keep if(sampleFlag==1) 
		
		// Lead everything a bunch, so that the first observation of every claimDy will have sequence
		bysort claimDySwID (t0): gen tempPanel = _n
		xtset claimDySwID tempPanel
		
		sum numTranMax		// find maximum number of transitions, to figure out many sequence vars you need
		local max = `r(max)'
		forvalues l=0/`max'{
			gen seq`l' = F`l'.stage
		}
		
		// THEN, convert everything to string using the phony sequence variable
		forvalues l=0/`max'{
			gen sgID`l' = "C" if(seq`l'==0)
				replace sgID`l' = "N" if(seq`l'==1)
				replace sgID`l' = "M" if(seq`l'==2)
				replace sgID`l' = "R" if(seq`l'==3)
				replace sgID`l' = "" if(seq`l'==.)
		}
		
		
		// Then, combine it all into one mega-string
		egen mega = concat(sgID*)		
		replace mega = trim(mega)
			// If it's resolved, then add an "R" to the end
			replace mega = mega + "R" if(ongoing==0)
	}

	// Tab and sort by frequency to see
	tab mega if(t0==0), sort
}
**************************
// Table 3 - Stage Information: Frequency, Spell Length
{
	cd "$dir"
	cd ..
	use "2 - h&a - continuous, no TVC.dta", clear

	// ## CHALLENGE ## //
		// How many times does a dispute-dyad, on average, appear in the challenge stage?
		local var stageCnt_chalMax
		sum `var' if(t0==0 & sampleFlag==1), detail	

		// What's the average length of time in the challenge stage, per spell?
		local var stageDuration
		sum stageDuration if(sampleFlag==1 & stage==0), detail
	
		
	// ## NEGOTIATION ## //
		// How many times does a dispute-dyad, on average, appear in the negot stage?
		local var stageCnt_negMax
		sum `var' if(t0==0 & sampleFlag==1), detail	

		// What's the average length of time in the negotiation stage, per spell?
		local var stageDuration
		sum stageDuration if(sampleFlag==1 & stage==1), detail
			
		
	// ## MIDs ## //	
		// How many times does a dispute-dyad, on average, appear in the mil stage?
		local var stageCnt_milMax
		sum `var' if(t0==0 & sampleFlag==1), detail	
		
		// What's the average length of time in the militarized stage, per spell?
		local var stageDuration
		sum stageDuration if(sampleFlag==1 & stage==2), detail
		
		
	// ## OVERALL ## //		
		// How many transitions does an average dispute-dyad go through?
		sum numTranM if(t0==0 & sampleFlag==1), detail
		
		// How long are these dispute-dyads lasting (including Grovers as two sep dispDys), in total?
		bysort claimDySwID (datetm): egen dispLength = max(t)
		sum dispLength if(t0==0 & sampleFlag==1), detail
	
}
**************************
// Table 4 - Multi-state Model of the Territorial Dispute Process
{
	* Generated in R. 
	* FILE: "MS Analysis_4 Stages.R"
	
	
	// Stata equivalent:
	cd "$dir"
	cd ..
	use "2 - h&a - continuous, no TVC.dta", clear
	
	stset t, enter(t0) f(status)  
	stcox milratioA- milratioI strvalueA- strvalueI tgtodiA- tgtodiI chgodiA- chgodiI netdemocA- netdemocI onetdemA- onetdemI, strata(trans) nohr efron	
		
		* Coefficients do not appear in same order as table.  To see which transition corresponds with which coefficient:
		qui{
			label define stgs 0 "C" 1 "N" 2 "M" 3 "R"
			label values stage stgs
			label values nextStage stgs
			
			tokenize `c(ALPHA)'
			forvalues a=1/9{
				noi di as gr "Transition ID: " as ye "``a''"
				noi tab stage nextStage if(transID=="``a''")
			}
		}
}
**************************
// Table 5 - Predicted Probability Comparison
{
	* Multi-state probabilities generated in R, using five-stage model.
	* FILE: "MS Analysis_5 Stages_TransProb.R"
	
	
	* H&A probabilities copied directly from book; see pages in table footer.
	
}
**************************
// Table 6 - H&A: ECU-PER, Maynas Region, 1/1950-8/1953
{
	cd "$dir"
	insheet using "4 - huth_allee_statusquodata.tab", clear
		format datetm  %tm
		label define stgs 0 "Challenge" 1 "Negotiations" 2 "Military"
		label values dvsq1 stgs
	
	list chgname tgtname datetm dvsq1 length msafinal if(dispno2==70 & datetm<=tm(1953m8)), noo sep(15)
}
**************************
// Table 7 - ECU-PER, Maynas Region, 1950 only (Dispute-Dyad-Month, Discrete)
{
	cd "$dir"
	cd ..
	use "1 - h&a - discrete.dta", clear
	label define stgs 0 "Challenge" 1 "Negotiations" 2 "Military" 3 "Resolved"
	label values stage stgs
		
	list chgname tgtname datetm stage if(dispno2==70 & datetm<=tm(1950m12)), noo sep(15)
}
**************************
// Table 8 - ECU-PER, Maynas Region, 1/1950-8/1953 (Dispute-Dyad-Stage Start Date, Continuous)
{
	cd "$dir"
	cd ..
	use "2 - h&a - continuous, no TVC.dta", clear
	label define stgs 0 "Challenge" 1 "Negotiations" 2 "Military" 3 "Resolved"
	label values stage stgs
	label values nextStage stgs
		
	list chalName tgtName datetm stage nextStage stageDuration if(dispno2==70 & datetm<=tm(1953m8) & status==1), noo sep(15)
}
**************************
// Table 9 - ECU-PER, Maynas Region, 1/1950-8/1953 (Dispute-Dyad-Stage Start Date-q, Continuous)
{
	cd "$dir"
	cd ..
	use "2 - h&a - continuous, no TVC.dta", clear
	label define stgs 0 "Challenge" 1 "Negotiations" 2 "Military" 3 "Resolved"
	label values stage stgs
	label values nextStage stgs
		
	list chalName tgtName datetm stage nextStage status stageDuration trans if(dispno2==70 & datetm<=tm(1953m8)), noo sep(3)
}
**************************
// Table 10 - Observed Transition Frequencies (five stages)
{
	cd "$dir"
	cd ..
	use "4 - h&a - continuous, disagg stages (5 stgs)", clear
	
	// numerator
	tab stage nextStage if(status==1)	
	
	// denominator for percentages
	qui{
		cap drop temp
		bysort claimDySwID t0 (nextStage): gen temp = _n==1
		forvalues s=1/5{
			noi di _n as gr "Stage " `s'
			noi count if(temp==1 & stage==`s')
			local denom = r(N)
			
			count if(status==1 & stage==`s')
			noi di as gr "TOTAL Col %: " as ye r(N)/`denom'*100
		}
		drop temp
	}
}
**************************
// Table 11 - Multi-state Model of the Territorial Dispute Process--5 Stages
{
	* Generated in R. 
	* FILE: "MS Analysis_5 Stages.R"
	
	
	// Stata equivalent:
	cd "$dir"
	cd ..
	use "4 - h&a - continuous, disagg stages (5 stgs)", clear
	
	stset t, enter(t0) f(status)  
	stcox 	milratio1- milratio12 ///
			strvalue1- strvalue12 ///
			tgtodi1- tgtodi12 ///
			chgodi1- chgodi12 ///
			netdemoc1- netdemoc12 ///
			onetdem1- onetdem12, ///
				strata(trans) nohr efron	
	
		* Coefficients do not appear in same order as table.  To see which transition corresponds with which coefficient:
		qui{
			label define stgs 1 "C" 2 "N" 3 "M" 4 "I" 5 "R"
			label values stage stgs
			label values nextStage stgs
			
			forvalues a=1/12{
				noi di as gr "Transition ID: " as ye "`a'"
				noi tab stage nextStage if(trans==`a')
			}
		}
}
**************************
// Table 12 - Comparison of Predictive Power
{
	// Multi-State Results
	** OBSERVED
	{
		// generate for our dataset (with the 398 dispute-dyads, to get 401 disp-dyad-occurrences)
		cd "$dir"
		cd ..
		use "1 - h&a - discrete.dta", clear		// to get our 401
		
		bysort claimDySwID (fakePanel2 datetm): gen duration = sum(temp)

		// to get the resolved disputes in, in an obvious way
		bysort claimDySwID (fakePanel2 datetm): egen max = max(duration)
		clonevar stageMod = stage
		bysort claimDySwID (fakePanel2 datetm): gen lastFlag = _n==_N
		bysort claimDySwID (fakePanel2 datetm): replace stageMod = 3 if(max<24 & lastFlag==1)
		
		tab stageMod if(duration==24 | (max<24 & lastFlag==1))
	}
	
	** PREDICTED
	{
		* Generated in R. 
		* FILE: "MS Analysis_5 Stages_Predict.R" [generates "Mstate_Predictions.dta"]
		
		// Results
		cd "$dir"
		cd ..
		use "Mstate_Predictions.dta", clear
		qui{
			rename pred predStg
				replace predStg = predStg - 1
			compress
		}
		tab predStg
	}
	
	** PRE CALCULATION
	{
		cd "$dir"
		cd ..
		use "Mstate_Predictions.dta", clear
		qui{
			rename pred predStg
				replace predStg = predStg - 1
			compress


			preserve
				tempfile theMerger

				use "1 - h&a - discrete.dta", clear

				*******************************************************************************
				// generate for our dataset (with the 398 dispute-dyads, to get 401 disp-dyad-occurrences)
				bysort claimDySwID (fakePanel2 datetm): gen duration = sum(temp)

					// to get the resolved disputes in, in an obvious way
					bysort claimDySwID (fakePanel2 datetm): egen max = max(duration)
					clonevar stageMod = stage
					bysort claimDySwID (fakePanel2 datetm): gen lastFlag = _n==_N
					bysort claimDySwID (fakePanel2 datetm): replace stageMod = 3 if(max<24 & lastFlag==1)

					
					keep if((lastFlag==1 & max<24) | duration==24)

				save `theMerger', replace

			restore

			merge claimDySwID using `theMerger', unique sort keep(stageMod)
			tab _m
			drop _m


			// Tab the stuff, and PRE.
			tab stageMod predStg
				local N = `r(N)'
				
			qui{
				// modal category
				modes stageMod, gen(test)
					sum stageMod if(test==1)
					local mode = `r(min)'
					drop test
				count if(stageMod==`mode')
				
				local modal = `r(N)'/`N'
				
				
				// % correctly predicted
				local correct = 0
				
				levelsof stageMod, local(stages)
				foreach s of local stages{
					count if(stageMod==`s' & predStg==`s')
					local correct = `correct' + `r(N)'
				}
				local pcp = `correct' / `N'
				
				
				// percent reduction in error
				local pre = (`pcp' - `modal') / (1 - `modal')

			noi di _n
			noi di as gr "Modal category: " %5.3f as ye `modal'*100 as gr "%"
			noi di as gr "Percent correctly predicted: " %5.3f as ye `pcp'*100 as gr "%"	
			noi di as gr "Percent reduction in error: " %5.3f as ye `pre'*100 as gr "%"
			}
		}
	}
	*
		
	
	// H&A
	** OBSERVED
	{
		// narrow it to H&A's 347, for the comparable bit.
		* based on how you coded H&A's simulations, keep the first observation from a dispute-dyad.
		cd "$dir"
		cd ..
		use "1 - h&a - discrete.dta", clear		// to get our 401
		
		bysort claimDySwID (fakePanel2 datetm): gen duration = sum(temp)

		// to get the resolved disputes in, in an obvious way
		bysort claimDySwID (fakePanel2 datetm): egen max = max(duration)
		clonevar stageMod = stage
		bysort claimDySwID (fakePanel2 datetm): gen lastFlag = _n==_N
		bysort claimDySwID (fakePanel2 datetm): replace stageMod = 3 if(max<24 & lastFlag==1)
		
		
		keep if(duration==24 | (max<24 & lastFlag==1))		//need this, to narrow to right subset of 24's.
		bysort dispno2 (claimDySwID fakePanel2 datetm): keep if _n==1	//now, look at every dispno2.  Keep the first instance of t=24, nuke the rest.


		tab stageMod
	}
	
	** PREDICTED
	qui{
		cd "$dir"
		insheet using "4 - huth_allee_statusquodata.tab", clear
			format datetm  %tm
			foreach x of varlist expratio-sptrprat{
				replace `x' = "" if(`x'=="NA")
				destring `x', replace
			}
			label define stgs 0 "Challenge" 1 "Negotiations" 2 "Military"
			label values dvsq1 stgs
			
		// Modified 7.4 (analogous to what we report as mstates)
		estsimp mlogit dvsq1 netdemoc onetdem ///
					milratio strvalue alliance tgtodi chgodi msafinal , b(0) robust  	
			

		// General rationale: Loop over each dispute in the dataset, and save the pred quantitites.
			
			local time = 24		//just to have it saved upfront.
			
			//start by creating variables
			gen dispno2_sim = .		//for the eventual trim and drop
			
			forvalues v=0/2{
				gen double pr`v' = .
				label variable pr`v' "Predicted probability of Stage `v' at t=`time'"
				
				gen double pr`v'_up = .
				label variable pr`v'_up "Upper 95% CI for Pr(Stage `v' at t=`time')"
						
				gen double pr`v'_lw = .
				label variable pr`v'_lw "Lower 95% CI for Pr(Stage `v' at t=`time')"
			}

			// for every dispute (of which there are 347), pull the quantities from 24 months in, and simulate.
			levelsof dispno2, local(disputes)
			
			sort dispno2 datetm
			gen counter = _n
			
			foreach d of local disputes{
				qui replace dispno2_sim = `d' in `d'
			
				// find the obsv number for this dispute's first observation
				qui sum counter if(dispno2==`d')
				local obsv = `r(min)'
				
				// set simulation's covariate values equal to those in the observation
				setx [`obsv']
				setx msafinal `time'
				
				// gen the pred quantities
				forvalues v=0/2{ 
					tempvar p`d'`v'
					
					qui simqi, prval(`v') genpr(`p`d'`v'')
					qui sum `p`d'`v'', mean
					qui return list
					qui replace pr`v'= r(mean) in `d'
					
					qui _pctile `p`d'`v'', p(2.5,97.5)
					qui return list
					qui replace pr`v'_lw = r(r1) in `d'
					qui replace pr`v'_up = r(r2) in `d'
				}
			}
			
		keep dispno2_sim pr*	
		order dispno2, first
		drop if(dispno2_sim==.)

		// which stage did the model predict?  Generate and see.
		gen predStg = 0 if(pr0==max(pr0,pr1,pr2))
			replace predStg = 1 if(pr1==max(pr0,pr1,pr2))
			replace predStg = 2 if(pr2==max(pr0,pr1,pr2))

		noi tab predStg // the verdict.	
	
	}
	** PRE CALCULATION
		// NOTE: continues from previous code block (H&A, Predicted).
	qui{
		preserve
			tempfile theMerger

			cd "$dir"
			cd ..
			use "1 - h&a - discrete.dta", clear

			*******************************************************************************
			// generate for our dataset (with the 398 dispute-dyads, to get 401 disp-dyad-occurrences)
			bysort claimDySwID (fakePanel2 datetm): gen duration = sum(temp)

				// to get the resolved disputes in, in an obvious way
				bysort claimDySwID (fakePanel2 datetm): egen max = max(duration)
				clonevar stageMod = stage
				bysort claimDySwID (fakePanel2 datetm): gen lastFlag = _n==_N
				bysort claimDySwID (fakePanel2 datetm): replace stageMod = 3 if(max<24 & lastFlag==1)

			********************************************************************************
			// narrow it to H&A's 347, for the comparable bit.
			* based on how you coded H&A's simulations, keep the first observation from a dispute-dyad.

			keep if(duration==24 | (max<24 & lastFlag==1))		//need this, to narrow to right subset of 24's.
			bysort dispno2 (claimDySwID fakePanel2 datetm): keep if _n==1	//now, look at every dispno2.  Keep the first instance of t=24, nuke the rest.

			clonevar dispno2_sim = dispno2
			save `theMerger', replace

		restore

		merge dispno2_sim using `theMerger', unique sort keep(stageMod)
		tab _m
		drop _m


		// Tab the stuff, and PRE.
		tab stageMod predStg
			local N = `r(N)'
			
		qui{
			// modal category
			modes stageMod, gen(test)
				sum stageMod if(test==1)
				local mode = `r(min)'
				drop test
			count if(stageMod==`mode')
			
			local modal = `r(N)'/`N'
			
			
			// % correctly predicted
			local correct = 0
			
			levelsof stageMod, local(stages)
			foreach s of local stages{
				count if(stageMod==`s' & predStg==`s')
				local correct = `correct' + `r(N)'
			}
			local pcp = `correct' / `N'
			
			
			// percent reduction in error
			local pre = (`pcp' - `modal') / (1 - `modal')

			
			noi di _n
			noi di as gr "Modal category: " %5.3f as ye `modal'*100 as gr "%"
			noi di as gr "Percent correctly predicted: " %5.3f as ye `pcp'*100 as gr "%"	
			noi di as gr "Percent reduction in error: " %5.3f as ye `pre'*100 as gr "%"
		}
	
	}
	
}
*

*******************************************************************************	
// FIGURES
**************************
// Figure 1 - The Stages of a Territorial Dispute
{
	* Generated by hand
}
**************************
// Figure 2 - The Stages of a Territorial Dispute
{
	* Generated by hand
}
**************************
// Figure 3 - The Effect of Challenger Regime Type on Dispute Resolution
{
	* Multi-state probabilities generated in R, using five-stage model.
	* FILE: "Figure 3 - 1, sims.R"  	[for the simulations]
	* FILE: "Figure 3 - 2, graph.R" 	[for the graph code]
	
	*** CAUTION: Simulation code takes ~1.5 days to run on Intel i5 3.30GHz, 8GB RAM
	  *		 	 If only interested in replicating graph, run graph code only.
	
}
